==> worker_3.4.1.orig.tar.bz2: worker-3.4.1/src/menutree.cc <==
/* menutree.cc
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2013 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "menutree.hh"

MenuTree::MenuTree() : m_root( "root", "", "" ),
                       m_current_node( &m_root )
{
}

MenuTreeNode *MenuTree::getRootNode()
{
    return &m_root;
}

void MenuTree::setFilter( const std::string &filter )
{
    m_filter = filter;
    m_matcher.setMatchString( filter );

    checkVisibility( m_current_node );
}

std::string MenuTree::createPath( const std::string &separator )
{
    return createPath( &m_root, m_current_node, separator );
}

std::string MenuTree::createPath( MenuTreeNode *node,
                                  const std::string &separator )
{
    return createPath( &m_root, node, separator );
}

std::string MenuTree::createPath( MenuTreeNode *me, MenuTreeNode *node,
                                  const std::string &separator )
{
    std::string res;

    if ( me == node ) {
        res = me->getName();
        return res;
    }

    for ( const auto it : *me ) {
        res = createPath( it, node, separator );

        if ( ! res.empty() ) {
            if ( me->getParent() != NULL ) {
                std::string res2 = me->getName();
                res2 += separator;
                res2 += res;
                return res2;
            } else {
                return res;
            }
        }
    }
    return "";
}

void MenuTree::setCurrentNode( MenuTreeNode *node )
{
    m_current_node = node;
}

MenuTreeNode *MenuTree::getCurrentNode() const
{
    return m_current_node;
}

void MenuTree::fillFullname( std::list< std::pair< std::string, MenuTreeNode * > > &entries )
{
    fillFullname( m_current_node, "", entries );
}

void MenuTree::fillFullname( MenuTreeNode *node,
                             const std::string &prefix,
                             std::list< std::pair< std::string, MenuTreeNode * > > &entries )
{
    for ( const auto it1 : *node ) {
        std::string new_prefix = prefix;

        new_prefix += it1->getName();

        if ( it1->hasChildren() ) {
            new_prefix += "/";
        }

        if ( it1->getMatched() ) {
            entries.push_back( std::make_pair( new_prefix, it1 ) );
        }

        fillFullname( it1,
                      new_prefix,
                      entries );
    }
}

void MenuTree::checkVisibility( MenuTreeNode *node,
                                bool root )
{
    if ( root ) node->clearMatchedFlag();

    if ( root && m_filter.empty() ) {
        for ( const auto it1 : *node ) {
            it1->setMatched( true );
        }
    }

    if ( ! m_filter.empty() ) {
        for ( const auto it1 : *node ) {
            if ( m_matcher.match( it1->getName() ) ) {
                it1->setMatched( true );
                checkVisibility( it1, false );
            } else {
                it1->setMatched( false );
                checkVisibility( it1, false );
            }
        }
    }
}

==> worker_3.8.2.orig.tar.bz2: worker-3.8.2/src/menutree.cc <==
/* menutree.cc
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2013-2015 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "menutree.hh"

MenuTree::MenuTree() : m_root( "root", "", "" ),
                       m_current_node( &m_root )
{
}

MenuTreeNode *MenuTree::getRootNode()
{
    return &m_root;
}

int MenuTree::setFilter( const std::string &filter )
{
    m_filter = filter;
    m_matcher.setMatchString( filter );

    return checkVisibility( m_current_node );
}

std::string MenuTree::createPath( const std::string &separator )
{
    return createPath( &m_root, m_current_node, separator );
}

std::string MenuTree::createPath( MenuTreeNode *node,
                                  const std::string &separator )
{
    return createPath( &m_root, node, separator );
}

std::string MenuTree::createPath( MenuTreeNode *me, MenuTreeNode *node,
                                  const std::string &separator )
{
    std::string res;

    if ( me == node ) {
        res = me->getName();
        return res;
    }

    for ( const auto it : *me ) {
        res = createPath( it, node, separator );

        if ( ! res.empty() ) {
            if ( me->getParent() != NULL ) {
                std::string res2 = me->getName();
                res2 += separator;
                res2 += res;
                return res2;
            } else {
                return res;
            }
        }
    }
    return "";
}

void MenuTree::setCurrentNode( MenuTreeNode *node )
{
    m_current_node = node;
}

MenuTreeNode *MenuTree::getCurrentNode() const
{
    return m_current_node;
}

void MenuTree::fillFullname( std::list< std::pair< std::string, MenuTreeNode * > > &entries )
{
    fillFullname( m_current_node, "", entries );
}

void MenuTree::fillFullname( MenuTreeNode *node,
                             const std::string &prefix,
                             std::list< std::pair< std::string, MenuTreeNode * > > &entries )
{
    for ( const auto it1 : *node ) {
        std::string new_prefix = prefix;

        new_prefix += it1->getName();

        if ( it1->hasChildren() ) {
            new_prefix += "/";
        }

        if ( it1->getMatched() ) {
            entries.push_back( std::make_pair( new_prefix, it1 ) );
        }

        fillFullname( it1,
                      new_prefix,
                      entries );
    }
}

int MenuTree::checkVisibility( MenuTreeNode *node,
                               bool root )
{
    if ( root ) node->clearMatchedFlag();

    int visible_elements = 0;

    if ( root && m_filter.empty() ) {
        for ( const auto it1 : *node ) {
            it1->setMatched( true );
            visible_elements++;
        }
    }

    if ( ! m_filter.empty() ) {
        for ( const auto it1 : *node ) {
            if ( m_matcher.match( it1->getName() ) ) {
                it1->setMatched( true );
                visible_elements++;
                
                visible_elements += checkVisibility( it1, false );
            } else {
                it1->setMatched( false );
                visible_elements += checkVisibility( it1, false );
            }
        }
    }

    return visible_elements;
}
